# encoding: utf-8

module VisiQuery
  class Client
    VisiBaseHost = "visibase.uplink.io"
    VisiBasePort = 80

    def initialize host = VisiBaseHost, port = VisiBasePort
      @http = Net::HTTP.new host, port
      @http.start
    end

    # Get a user by instance id, profile id, or username.
    def user identifier
      response = @http.get "/users/#{URI.escape identifier}.json"

      if response.code.to_i == 200
        body = ::MultiJson.load response.body

        if message = body['error']
          raise VisiBaseError, message
        else
          user = User.from_json body['accumulated']
        end
      end
    end

    # Get a ban reason by instance id, profile id, or username.
    def ban_reason identifier
      response = @http.get "/bans/#{URI.escape identifier}.json"

      if response.code.to_i == 200
        body = ::MultiJson.load response.body

        if message = body['error']
          raise VisiBaseError, message
        else
          user = BanReason.from_json body['accumulated']
        end
      end
    end

    # Publish changes to an object.
    def publish object
      case object
      when User
        publish_to '/users/create.json', :user, object
      when BanReason
        publish_to '/bans/create.json', :ban, object
      end
    end

    def close
      @http.finish
    end

  private

    def publish_to path, param, object
      response = @http.post path, url_encode(param, object)

      if response.code.to_i == 200
        body = ::MultiJson.dump response.body

        unless body['error']
          return object
        else
          raise VisiBaseError, body['error']
        end
      else
        raise VisiBaseError, "Server returned #{response.code} #{response.message}"
      end
    end

    def url_encode param, object
      URI.encode_www_form object.to_hash.map { |k, v| ["#{param}[#{k}]", v] }
    end
  end
end
