# encoding: utf-8

module VisiQuery
  class User
    Attributes = %w{real_name profile_text hours age facebook_name skype_name website residence}

    attr_accessor :id, :name
    attr_accessor :created_at, :updated_at, :first_login, :last_login
    Attributes.each &method(:attr_accessor)

    def self.from_json body
      User.new(body['name'], body['profile_id']).tap do |user|
        Attributes.each do |attribute|
          user.instance_variable_set :"@#{attribute}", body[attribute]
        end

        user.created_at = DateTime.parse body['created_at']
        user.updated_at = DateTime.parse body['updated_at']

        if body['last_login']
          user.last_login = DateTime.parse body['last_login']
        end

        if body['first_login']
          user.first_login = DateTime.parse body['first_login']
        end
      end
    end

    def initialize name, id
      @id = id.to_i
      @name = name
    end

    RejectedVariables = %w{@created_at @updated_at}

    def to_hash
      variables = instance_variables.dup
      variables.delete_if { |variable| RejectedVariables.include? variable.to_s }

      Hash.new.tap do |result|
        variables.each do |variable|
          value = instance_variable_get variable

          result[variable.to_s[1..-1]] = value if value
        end

        result['name'] = @name if @name
        result['profile_id'] = @id
      end
    end
  end
end
